clc
%% Load country-specific data
country = 'Kazakhstan';
[yData Date] = getCountryData( country, 2 );

N        = 18.8e6;  % Stock population
NOld     = 0.08;    % Elderly population
kappa    = 0.985;    % Recovery rate
kappaOld = 1 - 0.03; % Elderly recovery rate

yData = yData(52:52+108,:);

for option = 1:2
   %% Time specific parameters
   dataLength = size( yData, 1 );   %
   intComp    = 7;     % Interval in days
   predTime   = 20;    % Future predictions in days
   tSpan      = [1:intComp+1]';
   dayArray   = 1:intComp:dataLength;
   
   %% Parameters and Initial Conditions
   % Parameters boundaries
   if option == 2
      %     alpha beta    gamma   delta   kappa   kappa_old
      ub = [1/2    1       1/1     1/1     0.995    0.99];
      lb = [1/14   0.01    1/60    1/60    0.995    0.99];
   else
      %     alpha beta    gamma   delta   kappa   kappa_old
      ub = [1/5.1   1       1/18.8    1/18.8    0.995    0.99];
      lb = [1/5.1   0.01    1/18.8    1/18.8    0.995    0.99];
   end
   
   % Initial guess
   R_0   = 3;
   alpha = 1/5.1;
   gamma = 1/18.8;
   delta = gamma;
   phi   = delta * ( (1 - kappaOld)*NOld + (1 - kappa)*(1-NOld) );
   beta  = R_0 * (gamma + phi);
   
   % Initial conditions
   I0 = 2;
   E0 = 20*I0;
   R0 = 0;
   D0 = 0;
   S0 = N - E0 - I0 - R0 - D0;
   x0 = [S0 E0 I0 R0 D0];
   
   %% Execute algorithm
   runNonlinear;
end

%% Analyse estimation error
load modelStatic;
modelStatic = newModel;
load modelBounded;
modelBounded = newModel;
load modelAdHoc;
modelAdHoc = newModel;

figure; 
plot(modelStatic(1:109,1)-7, abs(Infected - modelStatic(1:109,2)), '-k', 'LineWidth', 1);
hold on;
plot(modelBounded(1:109,1)-7, abs(Infected - modelBounded(1:109,2)), '-.k', 'LineWidth', 1);
plot(modelAdHoc(1:109,1), abs(Infected - modelAdHoc(1:109,3)), ':k', 'LineWidth', 1);
xlabel('Days');
ylabel('Model Fitting Error (Active Confirmed Cases)');
xlim([0 109]);
grid;
legend('Simulation 1', 'Simulation 2', 'Simulation 3');

figure; 
plot(modelStatic(1:109,1)-7, abs(Deaths - modelStatic(1:109,3)), '-k', 'LineWidth', 1);
hold on;
plot(modelBounded(1:109,1)-7, abs(Deaths - modelBounded(1:109,3)), '-.k', 'LineWidth', 1);
plot(modelAdHoc(1:109,1), abs(Deaths - modelAdHoc(1:109,4)), ':k', 'LineWidth', 1);
xlabel('Days');
ylabel('Model Fitting Error (Cumulative Deaths)');
xlim([0 109]);
grid;
legend('Simulation 1', 'Simulation 2', 'Simulation 3');

figure;
errStatic = abs(Infected - modelStatic(1:109,2));
errBounded = abs(Infected - modelBounded(1:109,2));
errAdHoc = abs(Infected - modelAdHoc(1:109,3));
errInf = table(errStatic, errBounded, errAdHoc, 'VariableNames', {'Sim1', 'Sim2', 'Sim3'});
boxplot([errInf.Sim1 errInf.Sim2 errInf.Sim3], 'Jitter', 1);
set(gca, 'XTickLabel', {'Sim 1', 'Sim 2', 'Sim 3'});
ylabel('Model Fitting Error (Active Confirmed Cases)');

figure;
errStatic = abs(Deaths - modelStatic(1:109,3));
errBounded = abs(Deaths - modelBounded(1:109,3));
errAdHoc = abs(Deaths - modelAdHoc(1:109,4));
errDea = table(errStatic, errBounded, errAdHoc, 'VariableNames', {'Sim1', 'Sim2', 'Sim3'});
boxplot([errDea.Sim1 errDea.Sim2 errDea.Sim3], 'Jitter', 1);
set(gca, 'XTickLabel', {'Sim 1', 'Sim 2', 'Sim 3'});
ylabel('Model Fitting Error (Cumulative Deaths)');

%% EOL
