%% Plot actual data
figure;
set( gcf, 'Position', [1 514 1800 510] );
set(groot, 'DefaultAxesTickLabelInterpreter', 'none');

Infected  = movmean(yData(1:dataLength,1) - yData(1:dataLength,3), 7);
Deaths    = yData(1:dataLength,2);
Recovered = yData(1:dataLength,3);

subplot( 1,3,1 ); hold on;
plot( Infected, '.', 'MarkerSize', 18, 'Linewidth', 1 );

subplot( 1,3,2 ); hold on;
plot( Deaths, 'o', 'MarkerSize', 8, 'Linewidth', 1, 'Color', colour(2,:) );

subplot( 1,3,3 ); hold on;
plot( Infected, '.', 'MarkerSize', 18, 'Linewidth', 1 );
plot( Deaths, 'o', 'MarkerSize', 8, 'Linewidth', 1, 'Color', colour(2,:) );

%% Main algorithm
R0(1)          = R_0;
tau_inc(1)     = 1/alpha;
tau_rec(1)     = 1/gamma;
tau_hosp(1)    = 1/delta;
kappaArr(1)    = kappa;
kappaOldArr(1) = kappaOld;

modelI = [];
modelD = [];
tSim = [];
for ii = 1:length( dayArray )-1
    b = [alpha beta gamma delta kappa kappaOld];
    if tSpan(end)+intComp > dataLength
        tSpan = [tSpan(1):length( Deaths )]';
    end
    
    modelfun = @covidNonlinearMdl;
    mdlFit = lsqcurvefit( @(b,tSpan) modelfun( b, tSpan, x0, [N NOld], 2 ), b, tSpan, [Infected(tSpan) Deaths(tSpan)], lb, ub );
    
    alpha = mdlFit(1);
    beta  = mdlFit(2);
    gamma = mdlFit(3);
    delta = mdlFit(4);
    kappa = mdlFit(5);
    kappaOld = mdlFit(6);
    Delta = delta * ( (1 - kappaOld)*NOld + (1 - kappa)*(1-NOld) );
    
    R0(ii+1) = beta/(gamma+Delta);
    tau_inc(ii+1) = 1/alpha;
    tau_rec(ii+1) = 1/gamma;
    tau_hosp(ii+1) = 1/delta;
    kappaArr(ii+1) = kappa;
    kappaOldArr(ii+1) = kappaOld;
    
    if tSpan(end) >= length( Deaths )
        tSpan = [tSpan; [tSpan(end)+1:tSpan(end)+predTime]'];
    end
    
    mdlOut = covidNonlinearMdl( mdlFit, tSpan, x0, [N NOld], 1 );
    
    subplot( 1,3,1 );
    plot( tSpan, mdlOut(:,3), '-k', 'Linewidth', 1);%, 'Color', colour(1,:));
    
    subplot( 1,3,2 );
    plot( tSpan, mdlOut(:,5), '-k', 'Linewidth', 1);%, 'Color', colour(2,:));
    
    subplot( 1,3,3 );
    plot( tSpan, mdlOut(:,1),'-xk', 'Linewidth', 2, 'MarkerSize', 5); %'Color', colour(3,:));
    plot( tSpan, mdlOut(:,2),':k', 'Linewidth', 1);%'Color', colour(4,:));
    plot( tSpan, mdlOut(:,3), '-.k', 'Linewidth', 1);%, 'Color', colour(1,:));
    plot( tSpan, mdlOut(:,4), '--k', 'Linewidth', 1);%, 'Color', colour(5,:));
    plot( tSpan, mdlOut(:,5), '-k', 'Linewidth', 1);%, 'Color', colour(2,:) );
    
    if ii == 1
        outModel1 = mdlOut(:,[3 5]);
    else
        outModel1 = [outModel1(1:end-1,:); mdlOut(:,[3 5])];
    end
    
    x0 = mdlOut(end,:);
    tSpan  = tSpan + intComp;
    modelI = [modelI; mdlOut(:,3)];
    modelD = [modelD; mdlOut(:,5)];
    tSim = [tSim; tSpan];
end

%% Export results in a table on Command Window
dataOut = round( [R0' tau_inc' tau_rec' tau_hosp']*100 )/100;
fitOut = table( dayArray', dataOut(:,1), dataOut(:,2), dataOut(:,3), dataOut(:,4), 'VariableNames', {'Day' 'R0' 'tau_inc' 'tau_rec' 'tau_hosp'} )

%%
subplot( 1,3,1 );
xlabel( 'Days' );
ylabel( 'Active Confirmed Cases' );
legend( 'Actual data', 'Model', 'Location', 'NorthWest' );
xlim( [0 inf] );
ylim( [0 inf] );
grid on;

subplot( 1,3,2 );
xlabel( 'Days' );
ylabel( 'Cumulative Deaths' );
legend( 'Actual data', 'Model', 'Location', 'NorthWest' );
xlim( [0 inf] );
ylim( [0 inf] );
grid on;

subplot( 1,3,3 );
xlabel( 'Days' );
ylabel( 'Cases' );
legend( 'Active confirmed cases (Data)', 'Cumulative deaths (Data)', ...
   'Susceptible (Model)', 'Exposed (Model)', ...
   'Active confirmed cases (Model)', 'Recovered (Model)', ...
   'Cumulative deaths (Model)', 'Location', 'SouthEast' );
xlim( [0 inf] );
grid on;
set(gca, 'YScale', 'log');

%% Export and save simulation results
newModel = [tSim modelI modelD];

ii = 1;
sizetSim = size(newModel, 1);
while(ii < sizetSim)
   if (newModel(ii,1) == newModel(ii+1,1))
      newModel(ii+1,:) = [];
      sizetSim = size(newModel, 1);
   end
   ii = ii + 1;
end

if option == 2
   save modelStatic newModel;
else
   save modelBounded newModel;
end